#  MIT License
#
#  Copyright (c) 2020 Dan <https://github.com/delivrance>
#  Copyright (c) 2021 Gegham Zakaryan <https://github.com/zakaryan2004>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.

import yaml
import logging
import re
from functools import partial
from pathlib import Path

from . import emojipedia, languages


class Plate:
    def __init__(self, root: str = "locales", locale: str = "en_US", fallback: str = None, separator: str = "|"):
        """Creates a new Plate instance.

        Parameters:
            root (str, optional):
                The folder root containing all locale sources.
                Defaults to "locales".

            locale (str, optional):
                The default locale.
                Defaults to "en_US"

            fallback (str, optional):
                The fallback locale used when the translated phrase keys have no value.
                Defaults to the value passed to the "locale" parameter, which is "en_US" by default.

            separator (str):
                The string used for delimiting pluralized phrases.
        """
        self.root = root
        self.locale = locale
        self.fallback = fallback or locale
        self.separator = separator

        self.locales = {
            locale: [getattr(languages, locale), None]
            for locale in filter(lambda x: not x.startswith("__"), vars(languages))
        }

        self._load()

    def set_locale(self, locale: str):
        """Sets a new default locale.

        Parameters:
            locale (str):
                The new default locale to set.
        """
        self._check_valid_locale(locale)
        self.locale = locale

    def get_translator(self, locale: str):
        """Retrieves a translator for a given locale.

        Parameters:
            locale (str):
                The locale to get the translator for.
        """
        self._check_valid_locale(locale)
        return partial(self, locale=locale)

    def __call__(self, key: str, locale: str = None, *, count: int = None, **kwargs) -> str:
        """Translate a phrase given a key and an optional locale.

        Parameters:
            key (str):
                Phrase key to translate.

            locale (str, optional):
                The destination locale code the phrase will be translated to.
                Defaults to the current default locale.

            count (int, optional):
                Special optional count argument used for pluralization.
                Only applicable in case phrases contain {count}.

            **kwargs:
                Extra optional keyword argument for string interpolations.
                Applicable to all phrases that contain placeholders.

        Returns:
            str: The translated phrase.
        """
        if locale is None:
            locale = self.locale
        else:
            self._check_valid_locale(locale)

        phrase = self.locales[locale][1].get(key)

        if not phrase:
            phrase = self.locales[self.fallback][1][key]

        try:
            if count is not None:
                phrase = self._format_plurals(phrase, locale, count, **kwargs)

            return phrase.format(**kwargs)
        except KeyError as e:
            raise KeyError('Missing interpolation value for key "{}"'.format(e.args[0])) from None

    def _check_valid_locale(self, locale: str):
        if locale not in self.locales:
            raise ValueError(
                'Invalid locale code "{}". Possible values are: {}'.format(
                    locale, ", ".join('"{}" ({})'.format(k, v[0]) for k, v in self.locales.items())
                )
            )

    def _load(self):
        for path in Path(self.root).iterdir():
            name = path.name

            if not name.endswith(".yaml"):
                logging.warning('Skipping unknown file "{}"'.format(name))
                continue

            locale = name.split(".")[0]

            self._check_valid_locale(locale)

            with open(str(path), encoding="utf-8") as f:
                try:
                    locale_data = yaml.safe_load(f)
                except yaml.YAMLError as e:
                    raise ValueError('Error in file "{}": {}'.format(name, e)) from None

            for k, v in locale_data.items():
                if isinstance(v, list):
                    locale_data[k] = "".join(v)

            for k, v in locale_data.items():
                with_emoji = v

                if with_emoji is None:
                    continue

                for text in re.findall(r":(\w+):", with_emoji):
                    if text.islower():
                        logging.warning(
                            'Emoji "{}" from "{}" in "{}" should be in upper case: "{}"'.format(
                                text, k, locale, text.upper()
                            )
                        )

                    emoji = getattr(emojipedia, text.upper(), None)

                    if emoji is None:
                        raise ValueError('"{}" in "{}" contains unknown emoji "{}"'.format(k, locale, text))

                    with_emoji = re.sub(":{}:".format(text), emoji, with_emoji)

                locale_data[k] = with_emoji

            self.locales[locale][1] = locale_data

        for locale in self.locales.copy():
            if self.locales[locale][1] is None:
                self.locales.pop(locale)

        fallback_keys = self.locales[self.fallback][1].keys()

        for locale, locale_data in self.locales.items():
            locale_data = locale_data[1]

            for key in fallback_keys:
                if key not in locale_data:
                    raise ValueError('Missing translation key "{}" from "{}"'.format(key, locale))

                if not locale_data[key]:
                    logging.warning('Empty translation phrase for key "{}" in "{}"'.format(key, locale))

            for key in locale_data:
                if key not in fallback_keys:
                    raise ValueError(
                        'The key "{}" from "{}" does not exist in fallback locale "{}"'.format(
                            key, locale, self.fallback
                        )
                    )

    def _format_plurals(self, phrase: str, locale: str, count: int, **kwargs) -> str:
        options = [option.strip() for option in phrase.split(self.separator)]

        # TODO: Add locales plural rules

        index = count if count < 3 else 2

        return options[index].format(count=count, **kwargs)



#  MIT License
#
#  Copyright (c) 2020 Dan <https://github.com/delivrance>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.

af_ZA = "Afrikaans - South Africa"
sq_AL = "Albanian - Albania"
ar_DZ = "Arabic - Algeria"
ar_BH = "Arabic - Bahrain"
ar_EG = "Arabic - Egypt"
ar_IQ = "Arabic - Iraq"
ar_JO = "Arabic - Jordan"
ar_KW = "Arabic - Kuwait"
ar_LB = "Arabic - Lebanon"
ar_LY = "Arabic - Libya"
ar_MA = "Arabic - Morocco"
ar_OM = "Arabic - Oman"
ar_QA = "Arabic - Qatar"
ar_SA = "Arabic - Saudi Arabia"
ar_SY = "Arabic - Syria"
ar_TN = "Arabic - Tunisia"
ar_AE = "Arabic - United Arab Emirates"
ar_YE = "Arabic - Yemen"
hy_AM = "Armenian - Armenia"
Cy_az_AZ = "Azeri (Cyrillic) - Azerbaijan"
Lt_az_AZ = "Azeri (Latin) - Azerbaijan"
eu_ES = "Basque - Basque"
be_BY = "Belarusian - Belarus"
bg_BG = "Bulgarian - Bulgaria"
ca_ES = "Catalan - Catalan"
zh_CN = "Chinese - China"
zh_HK = "Chinese - Hong Kong SAR"
zh_MO = "Chinese - Macau SAR"
zh_SG = "Chinese - Singapore"
zh_TW = "Chinese - Taiwan"
zh_CHS = "Chinese (Simplified)"
zh_CHT = "Chinese (Traditional)"
hr_HR = "Croatian - Croatia"
cs_CZ = "Czech - Czech Republic"
da_DK = "Danish - Denmark"
div_MV = "Dhivehi - Maldives"
nl_BE = "Dutch - Belgium"
nl_NL = "Dutch - The Netherlands"
en_AU = "English - Australia"
en_BZ = "English - Belize"
en_CA = "English - Canada"
en_CB = "English - Caribbean"
en_IE = "English - Ireland"
en_JM = "English - Jamaica"
en_NZ = "English - New Zealand"
en_PH = "English - Philippines"
en_ZA = "English - South Africa"
en_TT = "English - Trinidad and Tobago"
en_GB = "English - United Kingdom"
en_US = "English - United States"
en_ZW = "English - Zimbabwe"
et_EE = "Estonian - Estonia"
fo_FO = "Faroese - Faroe Islands"
fa_IR = "Farsi - Iran"
fi_FI = "Finnish - Finland"
fr_BE = "French - Belgium"
fr_CA = "French - Canada"
fr_FR = "French - France"
fr_LU = "French - Luxembourg"
fr_MC = "French - Monaco"
fr_CH = "French - Switzerland"
gl_ES = "Galician - Galician"
ka_GE = "Georgian - Georgia"
de_AT = "German - Austria"
de_DE = "German - Germany"
de_LI = "German - Liechtenstein"
de_LU = "German - Luxembourg"
de_CH = "German - Switzerland"
el_GR = "Greek - Greece"
gu_IN = "Gujarati - India"
he_IL = "Hebrew - Israel"
hi_IN = "Hindi - India"
hu_HU = "Hungarian - Hungary"
is_IS = "Icelandic - Iceland"
id_ID = "Indonesian - Indonesia"
it_IT = "Italian - Italy"
it_CH = "Italian - Switzerland"
ja_JP = "Japanese - Japan"
kn_IN = "Kannada - India"
kk_KZ = "Kazakh - Kazakhstan"
kok_IN = "Konkani - India"
ko_KR = "Korean - Korea"
ky_KZ = "Kyrgyz - Kazakhstan"
lv_LV = "Latvian - Latvia"
lt_LT = "Lithuanian - Lithuania"
mk_MK = "Macedonian (FYROM)"
ms_BN = "Malay - Brunei"
ms_MY = "Malay - Malaysia"
mr_IN = "Marathi - India"
mn_MN = "Mongolian - Mongolia"
nb_NO = "Norwegian (Bokmål) - Norway"
nn_NO = "Norwegian (Nynorsk) - Norway"
pl_PL = "Polish - Poland"
pt_BR = "Portuguese - Brazil"
pt_PT = "Portuguese - Portugal"
pa_IN = "Punjabi - India"
ro_RO = "Romanian - Romania"
ru_RU = "Russian - Russia"
sa_IN = "Sanskrit - India"
Cy_sr_SP = "Serbian (Cyrillic) - Serbia"
Lt_sr_SP = "Serbian (Latin) - Serbia"
sk_SK = "Slovak - Slovakia"
sl_SI = "Slovenian - Slovenia"
es_AR = "Spanish - Argentina"
es_BO = "Spanish - Bolivia"
es_CL = "Spanish - Chile"
es_CO = "Spanish - Colombia"
es_CR = "Spanish - Costa Rica"
es_DO = "Spanish - Dominican Republic"
es_EC = "Spanish - Ecuador"
es_SV = "Spanish - El Salvador"
es_GT = "Spanish - Guatemala"
es_HN = "Spanish - Honduras"
es_MX = "Spanish - Mexico"
es_NI = "Spanish - Nicaragua"
es_PA = "Spanish - Panama"
es_PY = "Spanish - Paraguay"
es_PE = "Spanish - Peru"
es_PR = "Spanish - Puerto Rico"
es_ES = "Spanish - Spain"
es_UY = "Spanish - Uruguay"
es_VE = "Spanish - Venezuela"
sw_KE = "Swahili - Kenya"
sv_FI = "Swedish - Finland"
sv_SE = "Swedish - Sweden"
syr_SY = "Syriac - Syria"
ta_IN = "Tamil - India"
tt_RU = "Tatar - Russia"
te_IN = "Telugu - India"
th_TH = "Thai - Thailand"
tr_TR = "Turkish - Turkey"
uk_UA = "Ukrainian - Ukraine"
ur_PK = "Urdu - Pakistan"
Cy_uz_UZ = "Uzbek (Cyrillic) - Uzbekistan"
Lt_uz_UZ = "Uzbek (Latin) - Uzbekistan"
vi_VN = "Vietnamese - Vietnam"



#  MIT License
#
#  Copyright (c) 2020 Dan <https://github.com/delivrance>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.

from setuptools import setup

with open("README.md", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="YAML_Plate",
    version="1.0.2",
    description="A fork of Plate (Internationalization library for Python), for YAML",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/zakaryan2004/yaml_plate",
    author="zakaryan2004",
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development",
        "Topic :: Software Development :: Internationalization",
        "Topic :: Software Development :: Localization",
    ],
    keywords="i18n internationalization l10n localization library python yaml interpolation pluralization emoji",
    project_urls={
        "Tracker": "https://github.com/zakaryan2004/yaml_plate/issues",
        "Source": "https://github.com/zakaryan2004/yaml_plate",
    },
    python_requires="~=3.5",
    packages=["yaml_plate"],
    zip_safe=False,
)
